﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace WpfAudioPlayer.Models.Wmf
{
	/// <summary>
	/// Windows Media Format 系 API の Wrapper クラス。
	/// </summary>
	class NativeMethods
	{
		/// <summary>
		/// ASF のメタデータを操作する為のエディタを生成する。
		/// </summary>
		/// <param name="editor">生成したエディタを返す。</param>
		[DllImport( "WMVCore.dll", EntryPoint = "WMCreateEditor", CharSet = CharSet.Unicode, ExactSpelling = true, PreserveSig = false, SetLastError = true )]
		public static extern void WMCreateEditor( [Out, MarshalAs( UnmanagedType.Interface )] out IWMMetadataEditor2 editor );

		/// <summary>
		/// 
		/// </summary>
		/// <param name="unknownReserved"></param>
		/// <param name="rights"></param>
		/// <param name="syncReader"></param>
		[DllImport( "WMVCore.dll", EntryPoint = "WMCreateSyncReader", CharSet = CharSet.Unicode, ExactSpelling = true, PreserveSig = false, SetLastError = true )]
		public static extern void WMCreateSyncReader( [In] IntPtr unknownReserved, [In] WMT_RIGHTS rights, [Out, MarshalAs( UnmanagedType.Interface )] out IWMSyncReader syncReader );

		/// <summary>
		/// 成功を示す値。
		/// </summary>
		public const int S_OK = 0x00000000;

		/// <summary>
		/// メモリの確保に失敗した事を示すエラー値。
		/// </summary>
		public const ulong E_OUTOFMEMORY = 0x8007000E;

		/// <summary>
		/// データが見つからなかった事を示すエラー値。
		/// </summary>
		public const int ASF_E_NOTFOUND = unchecked( ( int )0xC00D07F0 );

		/// <summary>
		/// サンプル データが見つからなかった事を示すエラー値。
		/// </summary>
		public const int NS_E_NO_MORE_SAMPLES = unchecked( ( int )0xC00D0BCF );

		/// <summary>
		/// ファイルが見つからなかった事を示すエラー値。
		/// </summary>
		public const ulong NS_E_FILE_OPEN_FAILED = 0xFFFFFFFFC00D001D;
	}

	/// <summary>
	/// Windows Media の種別を示す定数を定義したクラス。
	/// </summary>
	static class WmfMediaTypes
	{
		public static readonly Guid MEDIASUBTYPE_I420               = new Guid( "30323449-0000-0010-8000-00AA00389B71" );
		public static readonly Guid MEDIASUBTYPE_IYUV               = new Guid( "56555949-0000-0010-8000-00AA00389B71" );
		public static readonly Guid MEDIASUBTYPE_RGB1               = new Guid( "E436EB78-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_RGB24              = new Guid( "E436EB7D-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_RGB32              = new Guid( "E436EB7E-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_RGB4               = new Guid( "E436EB79-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_RGB555             = new Guid( "E436EB7C-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_RGB565             = new Guid( "E436EB7B-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_RGB8               = new Guid( "E436EB7A-524F-11CE-9F53-0020AF0BA770" );
		public static readonly Guid MEDIASUBTYPE_UYVY               = new Guid( "59565955-0000-0010-8000-00AA00389B71" );
		public static readonly Guid MEDIASUBTYPE_VIDEOIMAGE         = new Guid( "1D4A45F2-E5F6-4B44-8388-F0AE5C0E0C37" );
		public static readonly Guid MEDIASUBTYPE_YUY2               = new Guid( "32595559-0000-0010-8000-00AA00389B71" );
		public static readonly Guid MEDIASUBTYPE_YV12               = new Guid( "31313259-0000-0010-8000-00AA00389B71" );
		public static readonly Guid MEDIASUBTYPE_YVU9               = new Guid( "39555659-0000-0010-8000-00AA00389B71" );
		public static readonly Guid MEDIASUBTYPE_YVYU               = new Guid( "55595659-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMFORMAT_MPEG2Video             = new Guid( "E06D80E3-DB46-11CF-B4D1-00805F6CBBEA" );
		public static readonly Guid WMFORMAT_Script                 = new Guid( "5C8510F2-DEBE-4CA7-BBA5-F07A104F8DFF" );
		public static readonly Guid WMFORMAT_VideoInfo              = new Guid( "05589F80-C356-11CE-BF01-00AA0055595A" );
		public static readonly Guid WMFORMAT_WaveFormatEx           = new Guid( "05589F81-C356-11CE-BF01-00AA0055595A" );
		public static readonly Guid WMFORMAT_WebStream              = new Guid( "DA1E6B13-8359-4050-B398-388E965BF00C" );
		public static readonly Guid WMMEDIASUBTYPE_ACELPnet         = new Guid( "00000130-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_Base             = new Guid( "00000000-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_DRM              = new Guid( "00000009-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_MP3              = new Guid( "00000050-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_MP43             = new Guid( "3334504D-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_MP4S             = new Guid( "5334504D-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_MPEG2_VIDEO      = new Guid( "E06D8026-DB46-11CF-B4D1-00805F6CBBEA" );
		public static readonly Guid WMMEDIASUBTYPE_MSS1             = new Guid( "3153534D-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_MSS2             = new Guid( "3253534D-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_PCM              = new Guid( "00000001-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WebStream        = new Guid( "776257D4-C627-41CB-8F81-7AC7FF1C40CC" );
		public static readonly Guid WMMEDIASUBTYPE_WMAudio_Lossless = new Guid( "00000163-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMAudioV2        = new Guid( "00000161-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMAudioV7        = new Guid( "00000161-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMAudioV8        = new Guid( "00000161-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMAudioV9        = new Guid( "00000162-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMSP1            = new Guid( "0000000A-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMV1             = new Guid( "31564D57-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMV2             = new Guid( "32564D57-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMV3             = new Guid( "33564D57-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIASUBTYPE_WMVP             = new Guid( "50564D57-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIATYPE_Audio               = new Guid( "73647561-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIATYPE_FileTransfer        = new Guid( "D9E47579-930E-4427-ADFC-AD80F290E470" );
		public static readonly Guid WMMEDIATYPE_Image               = new Guid( "34A50FD8-8AA5-4386-81FE-A0EFE0488E31" );
		public static readonly Guid WMMEDIATYPE_Script              = new Guid( "73636D64-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMMEDIATYPE_Text                = new Guid( "9BBA1EA7-5AB2-4829-BA57-0940209BCF3E" );
		public static readonly Guid WMMEDIATYPE_Video               = new Guid( "73646976-0000-0010-8000-00AA00389B71" );
		public static readonly Guid WMSCRIPTTYPE_TwoStrings         = new Guid( "82F38A70-C29F-11D1-97AD-00A0C95EA850" );
	}

	/// <summary>
	/// Windows Media の属性名となる定数を定義したクラス。
	/// </summary>
	static class WmfAttributeNames
	{
		public const string WMDuration = "Duration";
		public const string WMBitrate = "Bitrate";
		public const string WMSeekable = "Seekable";
		public const string WMStridable = "Stridable";
		public const string WMBroadcast = "Broadcast";
		public const string WMProtected = "Is_Protected";
		public const string WMTrusted = "Is_Trusted";
		public const string WMSignature_Name = "Signature_Name";
		public const string WMHasAudio = "HasAudio";
		public const string WMHasImage = "HasImage";
		public const string WMHasScript = "HasScript";
		public const string WMHasVideo = "HasVideo";
		public const string WMCurrentBitrate = "CurrentBitrate";
		public const string WMOptimalBitrate = "OptimalBitrate";
		public const string WMHasAttachedImages = "HasAttachedImages";
		public const string WMSkipBackward = "Can_Skip_Backward";
		public const string WMSkipForward = "Can_Skip_Forward";
		public const string WMNumberOfFrames = "NumberOfFrames";
		public const string WMFileSize = "FileSize";
		public const string WMHasArbitraryDataStream = "HasArbitraryDataStream";
		public const string WMHasFileTransferStream = "HasFileTransferStream";
		public const string WMContainerFormat = "WM/ContainerFormat";
		public const string WMTitle = "Title";
		public const string WMAuthor = "Author";
		public const string WMDescription = "Description";
		public const string WMRating = "Rating";
		public const string WMCopyright = "Copyright";
		public const string WMUse_DRM = "Use_DRM";
		public const string WMDRM_Flags = "DRM_Flags";
		public const string WMDRM_Level = "DRM_Level";
		public const string WMUse_Advanced_DRM = "Use_Advanced_DRM";
		public const string WMDRM_KeySeed = "DRM_KeySeed";
		public const string WMDRM_KeyID = "DRM_KeyID";
		public const string WMDRM_ContentID = "DRM_ContentID";
		public const string WMDRM_IndividualizedVersion = "DRM_IndividualizedVersion";
		public const string WMDRM_LicenseAcqURL = "DRM_LicenseAcqURL";
		public const string WMDRM_V1LicenseAcqURL = "DRM_V1LicenseAcqURL";
		public const string WMDRM_HeaderSignPrivKey = "DRM_HeaderSignPrivKey";
		public const string WMDRM_LASignaturePrivKey = "DRM_LASignaturePrivKey";
		public const string WMDRM_LASignatureCert = "DRM_LASignatureCert";
		public const string WMDRM_LASignatureLicSrvCert = "DRM_LASignatureLicSrvCert";
		public const string WMDRM_LASignatureRootCert = "DRM_LASignatureRootCert";
		public const string WMAlbumArtist = "WM/AlbumArtist";
		public const string WMAlbumSortOrder = "WM/AlbumSortOrder";
		public const string WMAlbumTitle = "WM/AlbumTitle";
		public const string WMArtistSortOrder = "WM/ArtistSortOrder";
		public const string WMComments = "WM/Comments";
		public const string WMSetSubTitle = "WM/SetSubTitle";
		public const string WMTitleSortOrder = "WM/TitleSortOrder";
		public const string WMTrack = "WM/Track";
		public const string WMPromotionURL = "WM/PromotionURL";
		public const string WMAlbumCoverURL = "WM/AlbumCoverURL";
		public const string WMGenre = "WM/Genre";
		public const string WMYear = "WM/Year";
		public const string WMGenreID = "WM/GenreID";
		public const string WMMCDI = "WM/MCDI";
		public const string WMComposer = "WM/Composer";
		public const string WMLyrics = "WM/Lyrics";
		public const string WMTrackNumber = "WM/TrackNumber";
		public const string WMToolName = "WM/ToolName";
		public const string WMToolVersion = "WM/ToolVersion";
		public const string WMIsVBR = "IsVBR";
		public const string WMBannerImageType = "BannerImageType";
		public const string WMBannerImageData = "BannerImageData";
		public const string WMBannerImageURL = "BannerImageURL";
		public const string WMCopyrightURL = "CopyrightURL";
		public const string WMAspectRatioX = "AspectRatioX";
		public const string WMAspectRatioY = "AspectRatioY";
		public const string ASFLeakyBucketPairs = "ASFLeakyBucketPairs";
		public const string WMNSCName = "NSC_Name";
		public const string WMNSCAddress = "NSC_Address";
		public const string WMNSCPhone = "NSC_Phone";
		public const string WMNSCEmail = "NSC_Email";
		public const string WMNSCDescription = "NSC_Description";
		public const string WMWriter = "WM/Writer";
		public const string WMConductor = "WM/Conductor";
		public const string WMProducer = "WM/Producer";
		public const string WMDirector = "WM/Director";
		public const string WMContentGroupDescription = "WM/ContentGroupDescription";
		public const string WMSubTitle = "WM/SubTitle";
		public const string WMPartOfSet = "WM/PartOfSet";
		public const string WMProtectionType = "WM/ProtectionType";
		public const string WMVideoHeight = "WM/VideoHeight";
		public const string WMVideoWidth = "WM/VideoWidth";
		public const string WMVideoFrameRate = "WM/VideoFrameRate";
		public const string WMMediaClassPrimaryID = "WM/MediaClassPrimaryID";
		public const string WMMediaClassSecondaryID = "WM/MediaClassSecondaryID";
		public const string WMPeriod = "WM/Period";
		public const string WMCategory = "WM/Category";
		public const string WMPicture = "WM/Picture";
		public const string WMLyrics_Synchronised = "WM/Lyrics_Synchronised";
		public const string WMOriginalLyricist = "WM/OriginalLyricist";
		public const string WMOriginalArtist = "WM/OriginalArtist";
		public const string WMOriginalAlbumTitle = "WM/OriginalAlbumTitle";
		public const string WMOriginalReleaseYear = "WM/OriginalReleaseYear";
		public const string WMOriginalFilename = "WM/OriginalFilename";
		public const string WMPublisher = "WM/Publisher";
		public const string WMEncodedBy = "WM/EncodedBy";
		public const string WMEncodingSettings = "WM/EncodingSettings";
		public const string WMEncodingTime = "WM/EncodingTime";
		public const string WMAuthorURL = "WM/AuthorURL";
		public const string WMUserWebURL = "WM/UserWebURL";
		public const string WMAudioFileURL = "WM/AudioFileURL";
		public const string WMAudioSourceURL = "WM/AudioSourceURL";
		public const string WMLanguage = "WM/Language";
		public const string WMParentalRating = "WM/ParentalRating";
		public const string WMBeatsPerMinute = "WM/BeatsPerMinute";
		public const string WMInitialKey = "WM/InitialKey";
		public const string WMMood = "WM/Mood";
		public const string WMText = "WM/Text";
		public const string WMDVDID = "WM/DVDID";
		public const string WMWMContentID = "WM/WMContentID";
		public const string WMWMCollectionID = "WM/WMCollectionID";
		public const string WMWMCollectionGroupID = "WM/WMCollectionGroupID";
		public const string WMUniqueFileIdentifier = "WM/UniqueFileIdentifier";
		public const string WMModifiedBy = "WM/ModifiedBy";
		public const string WMRadioStationName = "WM/RadioStationName";
		public const string WMRadioStationOwner = "WM/RadioStationOwner";
		public const string WMPlaylistDelay = "WM/PlaylistDelay";
		public const string WMCodec = "WM/Codec";
		public const string WMDRM = "WM/DRM";
		public const string WMISRC = "WM/ISRC";
		public const string WMProvider = "WM/Provider";
		public const string WMProviderRating = "WM/ProviderRating";
		public const string WMProviderStyle = "WM/ProviderStyle";
		public const string WMContentDistributor = "WM/ContentDistributor";
		public const string WMSubscriptionContentID = "WM/SubscriptionContentID";
		public const string WMWMADRCPeakReference = "WM/WMADRCPeakReference";
		public const string WMWMADRCPeakTarget = "WM/WMADRCPeakTarget";
		public const string WMWMADRCAverageReference = "WM/WMADRCAverageReference";
		public const string WMWMADRCAverageTarget = "WM/WMADRCAverageTarget";
		public const string EarlyDataDelivery = "EarlyDataDelivery";
		public const string JustInTimeDecode = "JustInTimeDecode";
		public const string SingleOutputBuffer = "SingleOutputBuffer";
		public const string SoftwareScaling = "SoftwareScaling";
		public const string DeliverOnReceive = "DeliverOnReceive";
		public const string ScrambledAudio = "ScrambledAudio";
		public const string DedicatedDeliveryThread = "DedicatedDeliveryThread";
		public const string EnableDiscreteOutput = "EnableDiscreteOutput";
		public const string SpeakerConfig = "SpeakerConfig";
		public const string DynamicRangeControl = "DynamicRangeControl";
		public const string AllowInterlacedOutput = "AllowInterlacedOutput";
		public const string VideoSampleDurations = "VideoSampleDurations";
		public const string StreamLanguage = "StreamLanguage";
		public const string DeinterlaceMode = "DeinterlaceMode";
		public const string InitialPatternForInverseTelecine = "InitialPatternForInverseTelecine";
		public const string JPEGCompressionQuality = "JPEGCompressionQuality";
		public const string WatermarkCLSID = "WatermarkCLSID";
		public const string WatermarkConfig = "WatermarkConfig";
		public const string InterlacedCoding = "InterlacedCoding";
		public const string FixedFrameRate = "FixedFrameRate";
		public const string OriginalSourceFormatTag = "_SOURCEFORMATTAG";
		public const string OriginalWaveFormat = "_ORIGINALWAVEFORMAT";
		public const string EDL = "_EDL";
		public const string Complexity = "_COMPLEXITYEX";
		public const string DecoderComplexityRequested = "_DECODERCOMPLEXITYPROFILE";
		public const string ReloadIndexOnSeek = "ReloadIndexOnSeek";
		public const string StreamNumIndexObjects = "StreamNumIndexObjects";
		public const string FailSeekOnError = "FailSeekOnError";
		public const string PermitSeeksBeyondEndOfStream = "PermitSeeksBeyondEndOfStream";
		public const string UsePacketAtSeekPoint = "UsePacketAtSeekPoint";
		public const string SourceBufferTime = "SourceBufferTime";
		public const string SourceMaxBytesAtOnce = "SourceMaxBytesAtOnce";
		public const string VBREnabled = "_VBRENABLED";
		public const string VBRQuality = "_VBRQUALITY";
		public const string VBRBitrateMax = "_RMAX";
		public const string VBRBufferWindowMax = "_BMAX";
		public const string VBRPeak = "VBR Peak";
		public const string BufferAverage = "Buffer Average";
		public const string ComplexityMax = "_COMPLEXITYEXMAX";
		public const string ComplexityOffline = "_COMPLEXITYEXOFFLINE";
		public const string ComplexityLive = "_COMPLEXITYEXLIVE";
		public const string IsVBRSupported = "_ISVBRSUPPORTED";
		public const string NumPasses = "_PASSESUSED";
		public const string MusicSpeechClassMode = "MusicSpeechClassMode";
		public const string MusicClassMode = "MusicClassMode";
		public const string SpeechClassMode = "SpeechClassMode";
		public const string MixedClassMode = "MixedClassMode";
		public const string SpeechCaps = "SpeechFormatCap";
		public const string PeakValue = "PeakValue";
		public const string AverageLevel = "AverageLevel";
		public const string Fold6To2Channels3 = "Fold6To2Channels3";
		public const string FoldToChannelsTemplate = "Fold%luTo%luChannels%lu";
		public const string DeviceConformanceTemplate = "DeviceConformanceTemplate";
		public const string EnableFrameInterpolation = "EnableFrameInterpolation";
	}
}
